@@ -51,7 +51,9 @@ reduce_fraction(struct intel_fraction f)
         a = b;
         b = c;
     }
-    return (struct intel_fraction) { f.num / b, f.den / b };
+    return (struct intel_fraction) {
+        f.num / b, f.den / b
+    };
 }
 
 static VAStatus
@@ -60,7 +62,7 @@ clear_border(struct object_surface *obj_surface)
     int width[3], height[3], hstride[3], vstride[3]; /* in byte */
     int planes;
     unsigned char* p;
-    int i,j;
+    int i, j;
 
     if (obj_surface->border_cleared)
         return VA_STATUS_SUCCESS;
@@ -71,7 +73,7 @@ clear_border(struct object_surface *obj_surface)
         height[0] = obj_surface->orig_height;
         height[1] = obj_surface->orig_height / 2;
         hstride[0] = hstride[1] = obj_surface->width;
-        vstride[0]= obj_surface->height;
+        vstride[0] = obj_surface->height;
         vstride[1] = obj_surface->height / 2;
 
     } else {
@@ -139,7 +141,7 @@ intel_encoder_check_yuv_surface(VADriverContextP ctx,
 
     if (obj_surface->fourcc == VA_FOURCC_NV12 ||
         (VAProfileHEVCMain10 == profile &&
-        obj_surface->fourcc == VA_FOURCC_P010)) {
+         obj_surface->fourcc == VA_FOURCC_P010)) {
 
         unsigned int tiling = 0, swizzle = 0;
         dri_bo_get_tiling(obj_surface->bo, &tiling, &swizzle);
@@ -160,11 +162,11 @@ intel_encoder_check_yuv_surface(VADriverContextP ctx,
     rect.y = 0;
     rect.width = obj_surface->orig_width;
     rect.height = obj_surface->orig_height;
-    
+
     src_surface.base = (struct object_base *)obj_surface;
     src_surface.type = I965_SURFACE_TYPE_SURFACE;
     src_surface.flags = I965_SURFACE_FLAG_FRAME;
-    
+
     status = i965_CreateSurfaces(ctx,
                                  obj_surface->orig_width,
                                  obj_surface->orig_height,
@@ -177,7 +179,7 @@ intel_encoder_check_yuv_surface(VADriverContextP ctx,
     encode_state->input_yuv_object = obj_surface;
     assert(obj_surface);
     i965_check_alloc_surface_bo(ctx, obj_surface, 1, fourcc, SUBSAMPLE_YUV420);
-    
+
     dst_surface.base = (struct object_base *)obj_surface;
     dst_surface.type = I965_SURFACE_TYPE_SURFACE;
     dst_surface.flags = I965_SURFACE_FLAG_FRAME;
@@ -197,16 +199,16 @@ intel_encoder_check_yuv_surface(VADriverContextP ctx,
 
 static VAStatus
 intel_encoder_check_jpeg_yuv_surface(VADriverContextP ctx,
-                                VAProfile profile,
-                                struct encode_state *encode_state,
-                                struct intel_encoder_context *encoder_context)
+                                     VAProfile profile,
+                                     struct encode_state *encode_state,
+                                     struct intel_encoder_context *encoder_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct i965_surface src_surface, dst_surface;
     struct object_surface *obj_surface;
     VAStatus status;
     VARectangle rect;
-    int format=0, fourcc=0, subsample=0;
+    int format = 0, fourcc = 0, subsample = 0;
 
     /* releae the temporary surface */
     if (encoder_context->is_tmp_id) {
@@ -226,9 +228,9 @@ intel_encoder_check_jpeg_yuv_surface(VADriverContextP ctx,
     dri_bo_get_tiling(obj_surface->bo, &tiling, &swizzle);
 
     if (tiling == I915_TILING_Y) {
-        if( (obj_surface->fourcc==VA_FOURCC_NV12)  || (obj_surface->fourcc==VA_FOURCC_UYVY) ||
-            (obj_surface->fourcc==VA_FOURCC_YUY2)  || (obj_surface->fourcc==VA_FOURCC_Y800) ||
-            (obj_surface->fourcc==VA_FOURCC_RGBA)  || (obj_surface->fourcc==VA_FOURCC_444P) ) {
+        if ((obj_surface->fourcc == VA_FOURCC_NV12)  || (obj_surface->fourcc == VA_FOURCC_UYVY) ||
+            (obj_surface->fourcc == VA_FOURCC_YUY2)  || (obj_surface->fourcc == VA_FOURCC_Y800) ||
+            (obj_surface->fourcc == VA_FOURCC_RGBA)  || (obj_surface->fourcc == VA_FOURCC_444P)) {
             encoder_context->input_yuv_surface = encode_state->current_render_target;
             encode_state->input_yuv_object = obj_surface;
             return VA_STATUS_SUCCESS;
@@ -244,43 +246,43 @@ intel_encoder_check_jpeg_yuv_surface(VADriverContextP ctx,
     src_surface.type = I965_SURFACE_TYPE_SURFACE;
     src_surface.flags = I965_SURFACE_FLAG_FRAME;
 
-    switch( obj_surface->fourcc) {
+    switch (obj_surface->fourcc) {
 
-        case VA_FOURCC_YUY2:
-            fourcc = VA_FOURCC_YUY2;
-            format = VA_RT_FORMAT_YUV422;
-            subsample = SUBSAMPLE_YUV422H;
-            break;
+    case VA_FOURCC_YUY2:
+        fourcc = VA_FOURCC_YUY2;
+        format = VA_RT_FORMAT_YUV422;
+        subsample = SUBSAMPLE_YUV422H;
+        break;
 
-        case VA_FOURCC_UYVY:
-            fourcc = VA_FOURCC_UYVY;
-            format = VA_RT_FORMAT_YUV422;
-            subsample = SUBSAMPLE_YUV422H;
-            break;
+    case VA_FOURCC_UYVY:
+        fourcc = VA_FOURCC_UYVY;
+        format = VA_RT_FORMAT_YUV422;
+        subsample = SUBSAMPLE_YUV422H;
+        break;
 
-        case VA_FOURCC_Y800:
-            fourcc = VA_FOURCC_Y800;
-            format = VA_RT_FORMAT_YUV400;
-            subsample = SUBSAMPLE_YUV400;
-            break;
+    case VA_FOURCC_Y800:
+        fourcc = VA_FOURCC_Y800;
+        format = VA_RT_FORMAT_YUV400;
+        subsample = SUBSAMPLE_YUV400;
+        break;
 
-        case VA_FOURCC_444P:
-            fourcc = VA_FOURCC_444P;
-            format = VA_RT_FORMAT_YUV444;
-            subsample = SUBSAMPLE_YUV444;
-            break;
+    case VA_FOURCC_444P:
+        fourcc = VA_FOURCC_444P;
+        format = VA_RT_FORMAT_YUV444;
+        subsample = SUBSAMPLE_YUV444;
+        break;
 
-        case VA_FOURCC_RGBA:
-            fourcc = VA_FOURCC_RGBA;
-            format = VA_RT_FORMAT_RGB32;
-            subsample = SUBSAMPLE_RGBX;
-            break;
+    case VA_FOURCC_RGBA:
+        fourcc = VA_FOURCC_RGBA;
+        format = VA_RT_FORMAT_RGB32;
+        subsample = SUBSAMPLE_RGBX;
+        break;
 
-        default: //All other scenarios will have NV12 format
-            fourcc = VA_FOURCC_NV12;
-            format = VA_RT_FORMAT_YUV420;
-            subsample = SUBSAMPLE_YUV420;
-            break;
+    default: //All other scenarios will have NV12 format
+        fourcc = VA_FOURCC_NV12;
+        format = VA_RT_FORMAT_YUV420;
+        subsample = SUBSAMPLE_YUV420;
+        break;
     }
 
     status = i965_CreateSurfaces(ctx,
@@ -305,12 +307,12 @@ intel_encoder_check_jpeg_yuv_surface(VADriverContextP ctx,
 
     //The Y800 format is expected to be tiled.
     //Linear Y800 is a corner case and needs code in the i965_image_processing.
-    if(obj_surface->fourcc != VA_FOURCC_Y800){
+    if (obj_surface->fourcc != VA_FOURCC_Y800) {
         status = i965_image_processing(ctx,
-                                   &src_surface,
-                                   &rect,
-                                   &dst_surface,
-                                   &rect);
+                                       &src_surface,
+                                       &rect,
+                                       &dst_surface,
+                                       &rect);
         assert(status == VA_STATUS_SUCCESS);
     }
 
@@ -335,10 +337,14 @@ intel_encoder_check_brc_h264_sequence_parameter(VADriverContextP ctx,
     assert(seq_param);
 
     if (!seq_param->num_units_in_tick || !seq_param->time_scale) {
-        framerate = (struct intel_fraction) { 30, 1 };
+        framerate = (struct intel_fraction) {
+            30, 1
+        };
     } else {
         // for the highest layer
-        framerate = (struct intel_fraction) { seq_param->time_scale, 2 * seq_param->num_units_in_tick };
+        framerate = (struct intel_fraction) {
+            seq_param->time_scale, 2 * seq_param->num_units_in_tick
+        };
     }
     framerate = reduce_fraction(framerate);
 
@@ -417,7 +423,9 @@ intel_encoder_check_brc_vp8_sequence_parameter(VADriverContextP ctx,
 
     if (!encoder_context->brc.framerate[encoder_context->layer.num_layers - 1].num) {
         // for the highest layer
-        encoder_context->brc.framerate[encoder_context->layer.num_layers - 1] = (struct intel_fraction) { 30, 1 };
+        encoder_context->brc.framerate[encoder_context->layer.num_layers - 1] = (struct intel_fraction) {
+            30, 1
+        };
         encoder_context->brc.need_reset = 1;
     }
 
@@ -454,9 +462,13 @@ intel_encoder_check_brc_hevc_sequence_parameter(VADriverContextP ctx,
         return VA_STATUS_ERROR_INVALID_PARAMETER;
 
     if (!seq_param->vui_time_scale || !seq_param->vui_num_units_in_tick)
-        framerate = (struct intel_fraction) { 30, 1 };
+        framerate = (struct intel_fraction) {
+        30, 1
+    };
     else
-        framerate = (struct intel_fraction) { seq_param->vui_time_scale, seq_param->vui_num_units_in_tick };
+        framerate = (struct intel_fraction) {
+        seq_param->vui_time_scale, seq_param->vui_num_units_in_tick
+    };
     framerate = reduce_fraction(framerate);
 
     num_iframes_in_gop = 1;
@@ -624,9 +636,13 @@ intel_encoder_check_framerate_parameter(VADriverContextP ctx,
         return;
 
     if (misc->framerate & 0xffff0000)
-        framerate = (struct intel_fraction) { misc->framerate & 0xffff, misc->framerate >> 16 & 0xffff };
+        framerate = (struct intel_fraction) {
+        misc->framerate & 0xffff, misc->framerate >> 16 & 0xffff
+    };
     else
-        framerate = (struct intel_fraction) { misc->framerate, 1 };
+        framerate = (struct intel_fraction) {
+        misc->framerate, 1
+    };
     framerate = reduce_fraction(framerate);
 
     if (encoder_context->brc.framerate[temporal_id].num != framerate.num ||
@@ -797,9 +813,11 @@ intel_encoder_check_temporal_layer_structure(VADriverContextP ctx,
 
 static VAStatus
 intel_encoder_check_misc_parameter(VADriverContextP ctx,
-                                  struct encode_state *encode_state,
-                                  struct intel_encoder_context *encoder_context)
+                                   VAProfile profile,
+                                   struct encode_state *encode_state,
+                                   struct intel_encoder_context *encoder_context)
 {
+    struct i965_driver_data *i965 = i965_driver_data(ctx);
     VAStatus ret = VA_STATUS_SUCCESS;
 
     if (encode_state->misc_param[VAEncMiscParameterTypeQualityLevel][0] &&
@@ -808,9 +826,28 @@ intel_encoder_check_misc_parameter(VADriverContextP ctx,
         VAEncMiscParameterBufferQualityLevel* param_quality_level = (VAEncMiscParameterBufferQualityLevel*)pMiscParam->data;
         encoder_context->quality_level = param_quality_level->quality_level;
 
-        if (encoder_context->quality_level == 0)
-            encoder_context->quality_level = ENCODER_DEFAULT_QUALITY;
-        else if (encoder_context->quality_level > encoder_context->quality_range) {
+        if (encoder_context->quality_level == 0) {
+            switch (profile) {
+            case VAProfileH264ConstrainedBaseline:
+            case VAProfileH264Main:
+            case VAProfileH264High:
+            case VAProfileH264MultiviewHigh:
+            case VAProfileH264StereoHigh:
+                if (IS_SKL(i965->intel.device_info) ||
+                    IS_BXT(i965->intel.device_info))
+                    encoder_context->quality_level = ENCODER_DEFAULT_QUALITY_AVC;
+                break;
+
+            case VAProfileHEVCMain:
+            case VAProfileHEVCMain10:
+                encoder_context->quality_level = ENCODER_DEFAULT_QUALITY_HEVC;
+                break;
+
+            default:
+                encoder_context->quality_level = ENCODER_DEFAULT_QUALITY;
+                break;
+            }
+        } else if (encoder_context->quality_level > encoder_context->quality_range) {
             ret = VA_STATUS_ERROR_INVALID_PARAMETER;
             goto out;
         }
@@ -833,7 +870,7 @@ intel_encoder_check_avc_parameter(VADriverContextP ctx,
                                   struct intel_encoder_context *encoder_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct object_surface *obj_surface;	
+    struct object_surface *obj_surface;
     struct object_buffer *obj_buffer;
     VAEncPictureParameterBufferH264 *pic_param = (VAEncPictureParameterBufferH264 *)encode_state->pic_param_ext->buffer;
     VAEncSequenceParameterBufferH264 *seq_param = (VAEncSequenceParameterBufferH264 *)encode_state->seq_param_ext->buffer;
@@ -846,7 +883,7 @@ intel_encoder_check_avc_parameter(VADriverContextP ctx,
 
     obj_surface = SURFACE(pic_param->CurrPic.picture_id);
     assert(obj_surface); /* It is possible the store buffer isn't allocated yet */
-    
+
     if (!obj_surface)
         goto error;
 
@@ -857,6 +894,9 @@ intel_encoder_check_avc_parameter(VADriverContextP ctx,
     if (!obj_buffer || !obj_buffer->buffer_store || !obj_buffer->buffer_store->bo)
         goto error;
 
+    if (encode_state->num_slice_params_ext > encoder_context->max_slice_or_seg_num)
+        goto error;
+
     encode_state->coded_buf_object = obj_buffer;
 
     for (i = 0; i < 16; i++) {
@@ -877,7 +917,7 @@ intel_encoder_check_avc_parameter(VADriverContextP ctx,
         }
     }
 
-    for ( ; i < 16; i++)
+    for (; i < 16; i++)
         encode_state->reference_objects[i] = NULL;
 
     /*
@@ -906,17 +946,17 @@ intel_encoder_check_mpeg2_parameter(VADriverContextP ctx,
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     VAEncPictureParameterBufferMPEG2 *pic_param = (VAEncPictureParameterBufferMPEG2 *)encode_state->pic_param_ext->buffer;
-    struct object_surface *obj_surface;	
+    struct object_surface *obj_surface;
     struct object_buffer *obj_buffer;
     int i = 0;
-    
+
     obj_surface = SURFACE(pic_param->reconstructed_picture);
     assert(obj_surface); /* It is possible the store buffer isn't allocated yet */
-    
+
     if (!obj_surface)
         goto error;
-    
-    encode_state->reconstructed_object = obj_surface;    
+
+    encode_state->reconstructed_object = obj_surface;
     obj_buffer = BUFFER(pic_param->coded_buf);
     assert(obj_buffer && obj_buffer->buffer_store && obj_buffer->buffer_store->bo);
 
@@ -953,10 +993,10 @@ intel_encoder_check_mpeg2_parameter(VADriverContextP ctx,
             goto error;
 
         encode_state->reference_objects[i++] = obj_surface;
-    } else 
+    } else
         goto error;
 
-    for ( ; i < 16; i++)
+    for (; i < 16; i++)
         encode_state->reference_objects[i] = NULL;
 
     return VA_STATUS_SUCCESS;
@@ -967,8 +1007,8 @@ error:
 
 static VAStatus
 intel_encoder_check_jpeg_parameter(VADriverContextP ctx,
-                                  struct encode_state *encode_state,
-                                  struct intel_encoder_context *encoder_context)
+                                   struct encode_state *encode_state,
+                                   struct intel_encoder_context *encoder_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct object_buffer *obj_buffer;
@@ -993,8 +1033,8 @@ error:
 
 static VAStatus
 intel_encoder_check_vp8_parameter(VADriverContextP ctx,
-                                    struct encode_state *encode_state,
-                                    struct intel_encoder_context *encoder_context)
+                                  struct encode_state *encode_state,
+                                  struct intel_encoder_context *encoder_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     VAEncPictureParameterBufferVP8 *pic_param = (VAEncPictureParameterBufferVP8 *)encode_state->pic_param_ext->buffer;
@@ -1003,13 +1043,13 @@ intel_encoder_check_vp8_parameter(VADriverContextP ctx,
     struct object_buffer *obj_buffer;
     int i = 0;
     int is_key_frame = !pic_param->pic_flags.bits.frame_type;
- 
+
     obj_surface = SURFACE(pic_param->reconstructed_frame);
     assert(obj_surface); /* It is possible the store buffer isn't allocated yet */
-    
+
     if (!obj_surface)
         goto error;
-    
+
     encode_state->reconstructed_object = obj_surface;
     obj_buffer = BUFFER(pic_param->coded_buf);
     assert(obj_buffer && obj_buffer->buffer_store && obj_buffer->buffer_store->bo);
@@ -1048,7 +1088,7 @@ intel_encoder_check_vp8_parameter(VADriverContextP ctx,
         encode_state->reference_objects[i++] = obj_surface;
     }
 
-    for ( ; i < 16; i++)
+    for (; i < 16; i++)
         encode_state->reference_objects[i] = NULL;
 
     encoder_context->is_new_sequence = (is_key_frame && seq_param);
@@ -1067,11 +1107,11 @@ error:
 
 static VAStatus
 intel_encoder_check_hevc_parameter(VADriverContextP ctx,
-                                  struct encode_state *encode_state,
-                                  struct intel_encoder_context *encoder_context)
+                                   struct encode_state *encode_state,
+                                   struct intel_encoder_context *encoder_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct object_surface *obj_surface;	
+    struct object_surface *obj_surface;
     struct object_buffer *obj_buffer;
     VAEncPictureParameterBufferHEVC *pic_param = (VAEncPictureParameterBufferHEVC *)encode_state->pic_param_ext->buffer;
     VAEncSliceParameterBufferHEVC *slice_param;
@@ -1091,7 +1131,7 @@ intel_encoder_check_hevc_parameter(VADriverContextP ctx,
 
     obj_surface = SURFACE(pic_param->decoded_curr_pic.picture_id);
     assert(obj_surface); /* It is possible the store buffer isn't allocated yet */
-    
+
     if (!obj_surface)
         goto error;
 
@@ -1102,6 +1142,9 @@ intel_encoder_check_hevc_parameter(VADriverContextP ctx,
     if (!obj_buffer || !obj_buffer->buffer_store || !obj_buffer->buffer_store->bo)
         goto error;
 
+    if (encode_state->num_slice_params_ext > encoder_context->max_slice_or_seg_num)
+        goto error;
+
     encode_state->coded_buf_object = obj_buffer;
 
     for (i = 0; i < 15; i++) {
@@ -1122,7 +1165,7 @@ intel_encoder_check_hevc_parameter(VADriverContextP ctx,
         }
     }
 
-    for ( ; i < 15; i++)
+    for (; i < 15; i++)
         encode_state->reference_objects[i] = NULL;
 
     for (i = 0; i < encode_state->num_slice_params_ext; i++) {
@@ -1209,7 +1252,7 @@ intel_encoder_check_vp9_parameter(VADriverContextP ctx,
             encode_state->reference_objects[i++] = NULL;
     }
 
-    for ( ; i < 16; i++)
+    for (; i < 16; i++)
         encode_state->reference_objects[i] = NULL;
 
     encoder_context->is_new_sequence = (is_key_frame && seq_param);
@@ -1258,10 +1301,10 @@ intel_encoder_sanity_check_input(VADriverContextP ctx,
         vaStatus = intel_encoder_check_jpeg_yuv_surface(ctx, profile, encode_state, encoder_context);
         break;
     }
- 
+
     case VAProfileVP8Version0_3: {
         vaStatus = intel_encoder_check_vp8_parameter(ctx, encode_state, encoder_context);
-         if (vaStatus != VA_STATUS_SUCCESS)
+        if (vaStatus != VA_STATUS_SUCCESS)
             goto out;
         vaStatus = intel_encoder_check_yuv_surface(ctx, profile, encode_state, encoder_context);
         break;
@@ -1289,15 +1332,15 @@ intel_encoder_sanity_check_input(VADriverContextP ctx,
     }
 
     if (vaStatus == VA_STATUS_SUCCESS)
-        vaStatus = intel_encoder_check_misc_parameter(ctx, encode_state, encoder_context);
+        vaStatus = intel_encoder_check_misc_parameter(ctx, profile, encode_state, encoder_context);
 
-out:    
+out:
     return vaStatus;
 }
- 
+
 static VAStatus
-intel_encoder_end_picture(VADriverContextP ctx, 
-                          VAProfile profile, 
+intel_encoder_end_picture(VADriverContextP ctx,
+                          VAProfile profile,
                           union codec_state *codec_state,
                           struct hw_context *hw_context)
 {
@@ -1312,7 +1355,7 @@ intel_encoder_end_picture(VADriverContextP ctx,
 
     encoder_context->mfc_brc_prepare(encode_state, encoder_context);
 
-    if((encoder_context->vme_context && encoder_context->vme_pipeline)) {
+    if ((encoder_context->vme_context && encoder_context->vme_pipeline)) {
         vaStatus = encoder_context->vme_pipeline(ctx, profile, encode_state, encoder_context);
         if (vaStatus != VA_STATUS_SUCCESS)
             return vaStatus;
@@ -1338,7 +1381,7 @@ intel_encoder_context_destroy(void *hw_context)
     encoder_context->mfc_context_destroy(encoder_context->mfc_context);
 
     if (encoder_context->vme_context_destroy && encoder_context->vme_context)
-       encoder_context->vme_context_destroy(encoder_context->vme_context);
+        encoder_context->vme_context_destroy(encoder_context->vme_context);
 
     if (encoder_context->enc_priv_state) {
         free(encoder_context->enc_priv_state);
@@ -1362,7 +1405,7 @@ intel_encoder_get_status(VADriverContextP ctx, struct hw_context *hw_context, vo
     return VA_STATUS_ERROR_UNIMPLEMENTED;
 }
 
-typedef Bool (* hw_init_func)(VADriverContextP, struct intel_encoder_context *);
+typedef Bool(* hw_init_func)(VADriverContextP, struct intel_encoder_context *);
 
 static struct hw_context *
 intel_enc_hw_context_init(VADriverContextP ctx,
@@ -1387,6 +1430,7 @@ intel_enc_hw_context_init(VADriverContextP ctx,
     encoder_context->quality_level = ENCODER_DEFAULT_QUALITY;
     encoder_context->quality_range = 1;
     encoder_context->layer.num_layers = 1;
+    encoder_context->max_slice_or_seg_num = 1;
 
     if (obj_config->entrypoint == VAEntrypointEncSliceLP)
         encoder_context->low_power_mode = 1;
@@ -1396,7 +1440,7 @@ intel_enc_hw_context_init(VADriverContextP ctx,
     case VAProfileMPEG2Main:
         encoder_context->codec = CODEC_MPEG2;
         break;
-        
+
     case VAProfileH264ConstrainedBaseline:
     case VAProfileH264Main:
     case VAProfileH264High:
@@ -1404,23 +1448,22 @@ intel_enc_hw_context_init(VADriverContextP ctx,
 
         if (obj_config->entrypoint == VAEntrypointEncSliceLP)
             encoder_context->quality_range = ENCODER_LP_QUALITY_RANGE;
-        else if(IS_GEN9(i965->intel.device_info)){
+        else if (IS_GEN9(i965->intel.device_info)) {
             encoder_context->quality_level = ENCODER_DEFAULT_QUALITY_AVC;
             encoder_context->quality_range = ENCODER_QUALITY_RANGE_AVC;
-        }
-        else
+        } else
             encoder_context->quality_range = ENCODER_QUALITY_RANGE;
         break;
 
     case VAProfileH264StereoHigh:
     case VAProfileH264MultiviewHigh:
-        if(IS_GEN9(i965->intel.device_info)){
+        if (IS_GEN9(i965->intel.device_info)) {
             encoder_context->quality_level = ENCODER_DEFAULT_QUALITY_AVC;
             encoder_context->quality_range = ENCODER_QUALITY_RANGE_AVC;
         }
         encoder_context->codec = CODEC_H264_MVC;
         break;
-        
+
     case VAProfileJPEGBaseline:
         encoder_context->codec = CODEC_JPEG;
         break;
@@ -1434,6 +1477,11 @@ intel_enc_hw_context_init(VADriverContextP ctx,
     case VAProfileHEVCMain:
     case VAProfileHEVCMain10:
         encoder_context->codec = CODEC_HEVC;
+
+        encoder_context->quality_level = ENCODER_DEFAULT_QUALITY_HEVC;
+        encoder_context->quality_range = ENCODER_QUALITY_RANGE_HEVC;
+
+        encoder_context->max_slice_or_seg_num = I965_MAX_NUM_SLICE;
         break;
 
     case VAProfileVP9Profile0:
@@ -1460,6 +1508,11 @@ intel_enc_hw_context_init(VADriverContextP ctx,
             if (encoder_context->codec == CODEC_H264)
                 encoder_context->context_roi = 1;
         }
+        if (obj_config->attrib_list[i].type == VAConfigAttribEncMaxSlices) {
+            if (encoder_context->codec == CODEC_H264 ||
+                encoder_context->codec == CODEC_HEVC)
+                encoder_context->max_slice_or_seg_num = obj_config->attrib_list[i].value;
+        }
     }
 
     if (vme_context_init) {
